import java.net.*;
import oracle.xml.parser.v2.*;
import org.w3c.dom.*;
import org.xml.sax.*;
import java.util.Properties;
import java.io.*;

// Command-line tool to load an XML Document and test XPath expression on it
public final class TestXPath {
  public static void main(String[] args) throws Exception {
    String filename = null, proxy = null;
    for (int z=0;z < args.length; z++) {
      if (args[z].equals("-x")) {
        if (args.length > z + 1) proxy = args[++z];
        else errorExit("No proxy specified after -x option");
      }
      else if (filename  == null) filename  = args[z];
    }
    if (proxy != null) {
      Properties p = System.getProperties();
      p.put("proxySet","true");
      p.put("proxyHost",proxy);
      p.put("proxyPort","80");
    }
    if (filename != null) {
      System.out.println("Type an XPath expression to test and press [Enter]");
      System.out.println("To quit, press [Enter] without entering a path.");
      run(filename);
    }
    else System.out.println("usage: TestXPath [-x proxyserver] filename|URL");
  }
  private static void run(String filename) throws Exception {
    // Open a reader to handle the XPath command line
    BufferedReader r = new BufferedReader(new InputStreamReader(System.in));
    // Turn the user's string-based filename or URL into a URL object
    URL xmlurl = newURL(filename);
    // Remember the base of the filename/URL for printing in the cmdline prompt
    String baseFilename = xmlurl.getFile();
    baseFilename = baseFilename.substring(baseFilename.lastIndexOf('/')+1);
    int qmark = baseFilename.indexOf('?');
    if (qmark >= 0) baseFilename = baseFilename.substring(0,qmark);
    // Create a new XML Parser
    DOMParser d = new DOMParser();
    // Try to parse the XML document at the URL
    try { d.parse(xmlurl); }
    catch(SAXParseException spe) { errorExit(filename+" is not well-formed.");}
    catch(FileNotFoundException f){errorExit(filename+" not found.");}
    // Get the parsed document from the parser and cast to access selectNodes()
    XMLDocument xmldoc = (XMLDocument)d.getDocument();
    // Loop until user types return on a blank line
    while (processCommand(getCommand(r,baseFilename),xmldoc)){}
  }
  private static boolean processCommand(String XPath, XMLDocument xmldoc)
                                  throws Exception {
    XPath = XPath != null ? XPath.trim() : "";
    if (XPath.equals("")) return false;
    else if (XPath.equals("/")) xmldoc.print(System.out);
    else {
      NodeList nl = null;
      // Otherwise handle the matching nodes
      try {
        // Select nodes matching XPath
        nl = xmldoc.selectNodes(XPath);
      }
      catch (XSLException err) {
        System.out.println(err.getMessage());
        return true;
      }
      int found = nl.getLength();
      if (found > 0) {
        System.out.print("\n");
        // Loop over matches
        for (int z=0; z < found; z++) {
          XMLNode curNode = (XMLNode)nl.item(z);
          // Print the current node as XML Markup to the output
          curNode.print(System.out);
          // Print a new line after Text or Attribute nodes
          int curNodeType = curNode.getNodeType();
          if (curNodeType == Node.ATTRIBUTE_NODE ||
              curNodeType == Node.TEXT_NODE ||
              curNodeType == Node.CDATA_SECTION_NODE) {
            System.out.print("\n");
          }
        }
      }
      else { System.out.println("No matches for "+ XPath); }
    }
    return true;
  }
  private static String getCommand(BufferedReader br,String baseFilename)
                            throws Exception {
    // Print out the command-line prompt
    System.out.print("\n"+baseFilename+"> ");
    // Return with the user's input
    return br.readLine();
  }
  private static URL newURL(String filename) throws MalformedURLException
  {
    URL url = null;
    try {
       // First try to see if filename is *already* a valid URL
       url = new URL(filename);
    }
    // If not, then assume it's a "naked" filename and make a URL
    catch (MalformedURLException ex) {
      // Get the absolute path of the file
      String path = (new File(filename)).getAbsolutePath();
      // If directory separator character is not a forward slash, make it so.
      if (File.separatorChar != '/') {
        path = path.replace(File.separatorChar,'/');
      }
      // Add a leading slash if path doesn't start with one (e.g. E:/foo/bar)
      if (!path.startsWith("/")) {
        path = "/"+path;
      }
      // Construct the file URL
      url = new URL("file://" + path);
    }
    return url;
  }
  private static void errorExit(String msg) {
    System.err.println("TestXPath: "+msg);System.exit(1);
  }
}
